﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Research.Kinect.Nui;
using Coding4Fun.Kinect.Wpf;
using Kinect.Toolbox.Record;


namespace SkeletalTracking
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    /// 
    

    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        //Kinect Runtime
        Runtime nui;

        //Targets and skeleton controller
        SkeletonController exampleController;
        CustomController1 yourController1;
        CustomController2 yourController2;

        //Holds the currently active controller
        SkeletonController currentController;
       
        Dictionary<int, Target> targets = new Dictionary<int, Target>();
        SkeletonRecorder sr = new SkeletonRecorder();
        
        //Scaling constants
        public float k_xMaxJointScale = 1.5f;
        public float k_yMaxJointScale = 1.5f;

        int i;
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            SetupKinect();
            exampleController = new SkeletonController(this);
            yourController1 = new CustomController1(this);
            yourController2 = new CustomController2(this);
            currentController = exampleController;
            InitTargets();
            i = 0;
        }
        
        private void InitTargets()
        {
            targets.Add(1, new Target(target1, 1));
            targets.Add(2, new Target(target2, 2));
            targets.Add(3, new Target(target3, 3));
            targets.Add(4, new Target(target4, 4));
            targets.Add(5, new Target(target5, 5));
            currentController.controllerActivated(targets);
            Canvas.SetZIndex(target1, 100);
            Canvas.SetZIndex(target2, 100);
            Canvas.SetZIndex(target3, 100);
            Canvas.SetZIndex(target4, 100);
            Canvas.SetZIndex(target5, 100);
        }

        private void SetupKinect()
        {
            //if (Runtime.Kinects.Count == 0)
            //{
            //    this.Title = "No Kinect connected"; 
            //}
            //else
            //{
                /*Point to the save/load skeletal file location */
            System.Diagnostics.Debug.Print(Environment.CurrentDirectory + @"\..\..\LAB4");
                System.IO.Stream stream = new System.IO.FileStream(Environment.CurrentDirectory + @"\..\..\LAB4", System.IO.FileMode.OpenOrCreate);
                 /*Start recording (SkeletonRecorder sr is part of this class)*/
                //sr.Start(stream);
                /*
                //use first Kinect
                nui = Runtime.Kinects[0];

                //Initialize to do skeletal tracking
                nui.Initialize(RuntimeOptions.UseSkeletalTracking | RuntimeOptions.UseColor | RuntimeOptions.UseDepthAndPlayerIndex);

                //add event to receive skeleton data, commented out for playback
                //nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);

                //add event to receive video data
                nui.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_VideoFrameReady);

                //to experiment, toggle TransformSmooth between true & false and play with parameters            
                nui.SkeletonEngine.TransformSmooth = true;
                TransformSmoothParameters parameters = new TransformSmoothParameters();
                // parameters used to smooth the skeleton data
                parameters.Smoothing = 0.3f;
                parameters.Correction = 0.3f;
                parameters.Prediction = 0.4f;
                parameters.JitterRadius = 0.7f;
                parameters.MaxDeviationRadius = 0.2f;
                nui.SkeletonEngine.SmoothParameters = parameters;

                //Open the video stream
                nui.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
                
                //Force video to the background
                Canvas.SetZIndex(image1, -10000);
                */
                /*REPLAY INIT*/
                SkeletonReplay sRep = new SkeletonReplay(stream);
                sRep.SkeletonFrameReady += new EventHandler<ReplaySkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);
                sRep.Start();
               
            //}
        }

        void nui_VideoFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            //Automagically create BitmapSource for Video
            image1.Source = e.ImageFrame.ToBitmapSource();            
        }

        void nui_SkeletonFrameReady(object sender, ReplaySkeletonFrameReadyEventArgs e)// SkeletonFrameReadyEventArgs e)
        {

            /*Recorder code */
           //sr.Record(e.SkeletonFrame);

           /*Normal SkeletonFrameReady Handling*/
           //SkeletonFrame allSkeletons = e.SkeletonFrame;
            
           //get the first tracked skeleton
           //SkeletonData skeleton = (from s in allSkeletons.Skeletons
             //                        where s.TrackingState == SkeletonTrackingState.Tracked
             //                     select s).FirstOrDefault();

            /* Replaying code */
            ReplaySkeletonData skeleton = e.SkeletonFrame.Skeletons[0];
            for (int i = 0; i < skeleton.Joints.Count; i++)
            {
                switch (skeleton.Joints.ElementAt(i).ID)
                {
                    case(JointID.Head):
                        SetEllipsePosition(headEllipse, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.HandLeft):
                        SetEllipsePosition(leftEllipse, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.HandRight):
                        SetEllipsePosition(rightEllipse, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.ShoulderCenter):
                        SetEllipsePosition(shoulderCenter, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.ShoulderRight):
                        SetEllipsePosition(shoulderRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.ShoulderLeft):
                        SetEllipsePosition(shoulderLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.AnkleLeft):
                        SetEllipsePosition(ankleLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.AnkleRight):
                        SetEllipsePosition(ankleRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.FootRight):
                        SetEllipsePosition(footRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.FootLeft):
                        SetEllipsePosition(footLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.WristLeft):
                        SetEllipsePosition(wristLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.WristRight):
                        SetEllipsePosition(wristRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.KneeRight):
                        SetEllipsePosition(kneeRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.KneeLeft):
                        SetEllipsePosition(kneeLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.ElbowLeft):
                        SetEllipsePosition(elbowLeft, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.ElbowRight):
                        SetEllipsePosition(elbowRight, skeleton.Joints.ElementAt(i));
                        break;
                    case (JointID.HipCenter):
                        SetEllipsePosition(hipCenter, skeleton.Joints.ElementAt(i));
                        break;
                }
            }

            /*Normal SkeletonFrame handling*/
            /* 
            if(skeleton != null)
            {
                //set positions on our joints of interest (already defined as Ellipse objects in the xaml)
                SetEllipsePosition(headEllipse, skeleton.Joints[JointID.Head]);
                SetEllipsePosition(leftEllipse, skeleton.Joints[JointID.HandLeft]);
                SetEllipsePosition(rightEllipse, skeleton.Joints[JointID.HandRight]);
                SetEllipsePosition(shoulderCenter, skeleton.Joints[JointID.ShoulderCenter]);
                SetEllipsePosition(shoulderRight, skeleton.Joints[JointID.ShoulderRight]);
                SetEllipsePosition(shoulderLeft, skeleton.Joints[JointID.ShoulderLeft]);
                SetEllipsePosition(ankleRight, skeleton.Joints[JointID.AnkleRight]);
                SetEllipsePosition(ankleLeft, skeleton.Joints[JointID.AnkleLeft]);
                SetEllipsePosition(footLeft, skeleton.Joints[JointID.FootLeft]);
                SetEllipsePosition(footRight, skeleton.Joints[JointID.FootRight]);
                SetEllipsePosition(wristLeft, skeleton.Joints[JointID.WristLeft]);
                SetEllipsePosition(wristRight, skeleton.Joints[JointID.WristRight]);
                SetEllipsePosition(elbowLeft, skeleton.Joints[JointID.ElbowLeft]);
                SetEllipsePosition(elbowRight, skeleton.Joints[JointID.ElbowRight]);
                SetEllipsePosition(ankleLeft, skeleton.Joints[JointID.AnkleLeft]);
                SetEllipsePosition(footLeft, skeleton.Joints[JointID.FootLeft]);
                SetEllipsePosition(footRight, skeleton.Joints[JointID.FootRight]);
                SetEllipsePosition(wristLeft, skeleton.Joints[JointID.WristLeft]);
                SetEllipsePosition(wristRight, skeleton.Joints[JointID.WristRight]);
                SetEllipsePosition(kneeLeft, skeleton.Joints[JointID.KneeLeft]);
                SetEllipsePosition(kneeRight, skeleton.Joints[JointID.KneeRight]);
                SetEllipsePosition(hipCenter, skeleton.Joints[JointID.HipCenter]);
                currentController.processSkeletonFrame(skeleton, targets);

            }*/
             
        }

        private void SetEllipsePosition(Ellipse ellipse, Joint joint)
        {    
            var scaledJoint = joint.ScaleTo(640, 480, k_xMaxJointScale, k_yMaxJointScale);

            Canvas.SetLeft(ellipse, scaledJoint.Position.X - (double)ellipse.GetValue(Canvas.WidthProperty) / 2 );
            Canvas.SetTop(ellipse, scaledJoint.Position.Y - (double)ellipse.GetValue(Canvas.WidthProperty) / 2);
            Canvas.SetZIndex(ellipse, (int) -Math.Floor(scaledJoint.Position.Z*100));
            if (joint.ID == JointID.HandLeft || joint.ID == JointID.HandRight)
            {   
                byte val = (byte)(Math.Floor((joint.Position.Z - 0.8)* 255 / 2));
                ellipse.Fill = new SolidColorBrush(Color.FromRgb(val, val, val));
            }
        }



        private void Window_Closed(object sender, EventArgs e)
        {
            //Cleanup
            sr.Stop();
            nui.Uninitialize();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.D1)
            {
                currentController = exampleController;
                controllerText.Content = "Example Controller";
                currentController.controllerActivated(targets);
            }

            if (e.Key == Key.D2)
            {
                currentController = yourController1;
                controllerText.Content = "Controller 1";
                currentController.controllerActivated(targets);
            }

            if (e.Key == Key.D3)
            {
                currentController = yourController2;
                controllerText.Content = "Controller 2";
                currentController.controllerActivated(targets);
            }

        }
    }


}
